
MATKEYS = {
    "si": "cSi",
    "Si": "cSi",

    "sio2": "SiO2",
    "sin": "SiN",
    "ge": "Ge",

}


def matname(k):
    if k in MATKEYS:
        return MATKEYS[k]
    return k.capitalize()


MATERIALS = {
    "cSi": {
        'epsilon': 3.48**2},
    "SiO2": {'epsilon': 1.44**2},
    "SiN": {'epsilon': 2.0**2},
    "Ge": {'epsilon': 4.0**2},
    "Si": {'epsilon': 3.48**2},
    'PEC': {'epsilon': 500},
}




from ..constants import *
from ..utils import *
import gdsfactory as gf
from gdsfactory.cross_section import Section
from networkx import center
from numpy import cumsum
from gdsfactory.generic_tech import LAYER_STACK, LAYER

# import .utils as utils
# from layers import *


def mimo(west=0, east=0, south=0, north=0,
         l=2.0, w=2.0, wwg=.5, lwg=None, taper=0,
         wwg_west=None, wwg_east=None, wwg_south=None, wwg_north=None,
         wwg_layer=LAYER.WG,  # bbox_layer=LAYER.WAFER,
         design_layer=DESIGN_LAYER,
         **kwargs):
    design = gf.Component()
    c = gf.Component(**kwargs)
    if lwg is None:
        lwg = 4*wwg
    p = [(0, 0), (l, 0), (l, w), (0, w)]
    design.add_polygon(p,                       layer=design_layer)
    c.add_polygon(p,                       layer=wwg_layer)

    ld = [west,  east, south, north]
    for i, v, d in zip(range(4), ld, [w, w, l, l]):
        if type(v) is int:
            ld[i] = [(.5+j)*d/v for j in range(v)]
    lwwg = [wwg_west, wwg_east, wwg_south, wwg_north]
    for i, v in enumerate(lwwg):
        if v is None:
            v = wwg
        if type(v) is float or type(v) is int:
            lwwg[i] = [v]*len(ld[i])

    n = 0
    for (i, x, y, d, wwg, a) in zip(
        range(4),
        [0,  l, 0, 0],
        [0, 0, 0, w],
        ld,
        lwwg,
        [180, 0, -90, 90]
    ):
        for wwg, v in zip(wwg, d):
            center = (x, y+v) if i in [0, 1] else (x+v, y)
            wwg2 = wwg+2*taper*lwg
            name = "o"+str(n+1)
            design.add_port(name, center=center, width=wwg2,
                            orientation=a, layer=wwg_layer)
            wg = c << gf.components.taper(
                length=lwg, width1=wwg, width2=wwg2, layer=wwg_layer)
            wg.connect("o2", design.ports[name])
            c.add_port(name, port=wg.ports["o1"])
            n += 1

    design = c << design
    return c



# # -*- coding: utf-8 -*-
# """Convert an image file to a GDS file
# """

# import cv2
# import numpy as np
# import gdspy

# import argparse


# def minmax(v):
#     if v > 255:
#         v = 255
#     if v < 0:
#         v = 0
#     return v


# def main(fileName, sizeOfTheCell, layerNum, isDither, scale):
#     """Convert an image file (fileName) to a GDS file
#     """
#     print("Converting an image file to a GDS file..")
#     # Read an image file
#     img = cv2.resize(cv2.imread(fileName), dsize=None, fx=scale, fy=scale)

#     width = img.shape[1]
#     height = img.shape[0]
#     print("width:{0}".format(width))
#     print("height:{0}".format(height))

#     # Convert an image to grayscale one
#     gray = cv2.cvtColor(img, cv2.COLOR_RGB2GRAY)

#     if (isDither):
#         # Floyd–Steinberg dithering
#         # https://en.wikipedia.org/wiki/Floyd%E2%80%93Steinberg_dithering
#         for y in range(0, height-1):
#             for x in range(1, width-1):
#                 old_p = gray[y, x]
#                 new_p = np.round(old_p/255.0) * 255
#                 gray[y, x] = new_p
#                 error_p = old_p - new_p
#                 gray[y, x+1] = minmax(gray[y, x+1] + error_p * 7 / 16.0)
#                 gray[y+1, x-1] = minmax(gray[y+1, x-1] + error_p * 3 / 16.0)
#                 gray[y+1, x] = minmax(gray[y+1, x] + error_p * 5 / 16.0)
#                 gray[y+1, x+1] = minmax(gray[y+1, x+1] + error_p * 1 / 16.0)

#         ret, binaryImage = cv2.threshold(gray, 0, 255, cv2.THRESH_OTSU)
#     else:
#         ret, binaryImage = cv2.threshold(gray, 0, 255, cv2.THRESH_OTSU)

#     # Fill orthological corner
#     for x in range(width - 1):
#         for y in range(height - 1):
#             if binaryImage.item(y, x) == 0 and binaryImage.item(y + 1, x) == 255 \
#                     and binaryImage.item(y, x + 1) == 255 and binaryImage.item(y + 1, x + 1) == 0:
#                 # binaryImage.itemset((y + 1, x), 0)
#                 binaryImage[y + 1, x] = 0
#             elif binaryImage.item(y, x) == 255 and binaryImage.item(y + 1, x) == 0 \
#                     and binaryImage.item(y, x + 1) == 0 and binaryImage.item(y + 1, x + 1) == 255:
#                 # binaryImage.itemset((y + 1, x + 1), 0)
#                 binaryImage[y + 1, x+1] = 0

#     # Output image.bmp
#     cv2.imwrite("image.bmp", binaryImage)

#     # The GDSII file is called a library, which contains multiple cells.
#     lib = gdspy.GdsLibrary()
#     gdspy.current_library = gdspy.GdsLibrary()

#     # Geometry must be placed in cells.
#     unitCell = lib.new_cell('CELL')
#     square = gdspy.Rectangle((0.0, 0.0), (1.0, 1.0), layer=(int)(layerNum))
#     unitCell.add(square)

#     grid = lib.new_cell("GRID")

#     for x in range(width):
#         for y in range(height):
#             if binaryImage.item(y, x) == 0:
#                 # print("({0}, {1}) is black".format(x, y))
#                 cell = gdspy.CellReference(
#                     unitCell, origin=(x, height - y - 1))
#                 grid.add(cell)

#     scaledGrid = gdspy.CellReference(
#         grid, origin=(0, 0), magnification=(float)(sizeOfTheCell))

#     # Add the top-cell to a layout and save
#     top = lib.new_cell("TOP")
#     top.add(scaledGrid)
#     s = fileName.split(".")
#     s = (".").join(s[:-1])
#     lib.write_gds(f"{s}.gds")


# if __name__ == "__main__":
#     parser = argparse.ArgumentParser()
#     parser.add_argument('fileName', type=str,
#                         help='name of the input image file')
#     parser.add_argument('sizeOfTheCell', type=float,
#                         help='size of the unit-cells (minimum width and space) [um]')
#     parser.add_argument('layerNum', type=int,
#                         help='layer number of the output GDSII file')
#     parser.add_argument('--scale', default=1.0, type=float, help='scale')
#     parser.add_argument('-d', action='store_true',
#                         help='Floyd–Steinberg dithering')
#     args = parser.parse_args()

#     main(args.fileName, args.sizeOfTheCell, args.layerNum, args.d, args.scale)



from .setup import *
from ..constants import *
from ..layers import *
from ..utils import *
import gdsfactory as gf
from copy import deepcopy
import matplotlib.pyplot as plt
import numpy as np

from gdsfactory.cross_section import Section
from gdsfactory.generic_tech import LAYER_STACK, LAYER


def make_pic_sim_prob(path, c: gf.Component, nres,
                      wavelengths,
                      entries=None, keys=None,
                      layer_stack=LAYER_STACK,
                      study="sparams",

                      wavelength=None,
                      wl_res=.01,
                      **kwargs):
    if wavelength:
        raise ValueError("wavelength is deprecated, use wavelengths instead")

    ports = [p.name for p in c.get_ports_list(prefix="o")]

    if not entries:
        entries = []
        if not keys:
            keys = []
            for i in ports:
                for o in ports:
                    keys.append(f"o,i")

        if type(wavelengths) not in [list, tuple, np.ndarray]:
            wavelengths = [wavelengths]
        else:
            wavelengths = wavelengths
        wavelengths, center_wavelength, T = adjust_wavelengths(
            wavelengths, wl_res)
        for w in wavelengths:
            for k in keys:
                entries.append([w, *unpack_sparam_key(k)])

    l = []
    wavelengths = SortedSet()
    for w, po, mo, pi, mi in entries:
        k = [w, pi, mi, pi, mi]
        if k not in entries:
            l.append(k)
        wavelengths.add(w)
    entries.extend(l)
    wavelengths = list(wavelengths)

    imow = {}
    for w, po, mo, pi, mi in entries:
        if pi not in imow:
            imow[pi] = {}
        if mi not in imow[pi]:
            imow[pi][mi] = {}

        if po not in imow[pi][mi]:
            imow[pi][mi][po] = mo
        else:
            imow[pi][mi][po] = max(imow[pi][mi][po], mo)

    runs = []
    for _w in [1]:
        for i in imow:
            for mi in imow[i]:
                d = {
                    "sources": {
                        i: {
                            "center": (np.array(c.ports[i].center)/1e3).tolist(),
                            "width": (np.array(c.ports[i].width)/1e3).tolist(),
                            "normal": normal_from_orientation(c.ports[i].orientation),
                            "wavelength_mode_numbers": {w: [mi] for w in wavelengths},
                            "port": i,
                        }},
                    "monitors": {
                        o: {
                            "port": o,
                            "normal": normal_from_orientation(c.ports[o].orientation),
                            "center": (np.array(c.ports[o].center)/1e3).tolist(),
                            "width": (np.array(c.ports[o].width)/1e3).tolist(),
                            "wavelength_mode_numbers": {w: list(range(imow[i][mi][o]+1)) for w in wavelengths},
                        } for o in imow[i][mi]}}
                d["sources"] = SortedDict(d["sources"])
                d["monitors"] = SortedDict(d["monitors"])
                runs.append(d)

    prob = setup(path, c, study=study,  nres=nres, center_wavelength=center_wavelength,
                 runs=runs,
                 layer_stack=layer_stack, **kwargs)
    prob["wavelengths"] = wavelengths
    prob["Tss"] = T if len(wavelengths) > 1 else None
    save_prob(prob, path)
    return prob

    # l = [k for k in imow if port_number(k) == pi]
    # if not l:
    #     imow[f"o{pi}@{mi}"] = []
    # else:
    #     k = l[0]
    #     mn = max(mode_number(k), mi)
    #     if mn != mode_number(k):
    #         imow[i] = imow[k]
    #         del imow[k]

    # l = [k for k in imow[i] if port_number(k) == po]
    # if not l:
    #     imow[f"o{pi}@{mi}"]
    # else:
    #     k = l[0]
    #     mn = max(mode_number(k), mi)
    #     if mn != mode_number(k):
    #         imow[f"o{pi}@{mn}"] = imow[k]
    #         del imow[k]

    # if po not in imow[pi]:
    #     imow[pi]["o"][po] = mo
    # else:
    #     imow[pi]["o"][po] = max(imow[pi]["o"][po], mo)
